==> v0.3-beta.tar.gz: wonx/UNIXTimer.c <==
/*****************************************************************************/
/*                                                                   */
/*****************************************************************************/

#include <unistd.h>
#include <signal.h>

#include "UNIXTimerP.h"
#include "etc.h"

/*****************************************************************************/
/* дؿ                                                          */
/*****************************************************************************/

/*---------------------------------------------------------------------------*/
/* Хåؿ                                                          */
/*---------------------------------------------------------------------------*/

/* pointed_unix_timer ͤϤѲ뤫狼ʤΤǡŬػߤ */

volatile static UNIXTimer pointed_unix_timer = NULL;

static void UNIXTimer_CallBackFunction(int argument)
{
  int ret;

  /*
    static ʥե饰ΩƤơХåؿ饳Хåؿ
    ƤФ줿Τ򸡽ФΤɬפ⡥
    */

  if (pointed_unix_timer == NULL) return;
  if (!pointed_unix_timer->timer_on) return;

  if (pointed_unix_timer->pause) {
    pointed_unix_timer->interrupt_in_pause++;
  } else {
    if (pointed_unix_timer->interrupt_in_pause == 0)
      pointed_unix_timer->interrupt_in_pause = 1;
    while (pointed_unix_timer->interrupt_in_pause > 0) {
      pointed_unix_timer->interrupt_in_pause--;
      if (pointed_unix_timer->callback != NULL) {

	/*
	 * Хåؿ椫 UNIXTimer_Unpause() ʤɤƤФơ
	 * 餵˥ХåؿƤФ줿ꤷȤΤˡ
	 * ݡ롥
	 */
	pointed_unix_timer->pause++;

	ret = (*pointed_unix_timer->callback)(pointed_unix_timer->parameter);

	pointed_unix_timer->pause--;

	/*
	 * Хåؿ UNIXTimer_* ϢδؿƤӽФȡ
	 *  UNIXTimer ֥Ȥξ֤ѤäƤޤǽ롥
	 */

	/*
	 * ʲνƱȤؿƬǤäƤΤǡ
	 * Ŭƺʤ褦դ뤳ȡ
	 */
	if (pointed_unix_timer == NULL) return;
	if (!pointed_unix_timer->timer_on) return;

	if (ret) {
	  pointed_unix_timer->interrupt_in_pause = 0;
	  UNIXTimer_OFF(pointed_unix_timer);
	  return;
	}

	/*
	 * Хåؿǡݡξ֤ڤؤäƤޤä
	 * ȤΤᡥ
	 * ʲνƱ褦ʤȤäˤäƤΤǡ
	 * Ŭƺʤ褦դ뤳ȡ
	 */
	if (pointed_unix_timer->pause) break;

      }
    }
  }

  if (pointed_unix_timer->auto_preset)
    UNIXTimer_ON(pointed_unix_timer);
  else
    UNIXTimer_OFF(pointed_unix_timer);

  return;
}

/*---------------------------------------------------------------------------*/
/* ޤ ON, OFF                                                          */
/*---------------------------------------------------------------------------*/

/*
 * Τνϡ˥޳ߤȯʤɤ
 * դ뤳ȡǤʤԲĲʥХθˤʤ롥
 */

int UNIXTimer_ON(UNIXTimer unix_timer)
{
  unix_timer->timer_on = 1;
  pointed_unix_timer = unix_timer;
  ualarm(unix_timer->interval, 0);
  return (0);
}

int UNIXTimer_OFF(UNIXTimer unix_timer)
{
  alarm(0); /* ޤ̵ˤ */
  unix_timer->timer_on = 0;
  pointed_unix_timer = NULL;
  return (0);
}

int UNIXTimer_IsON(UNIXTimer unix_timer)
{ return (unix_timer->timer_on != 0); }
int UNIXTimer_IsOFF(UNIXTimer unix_timer)
{ return (unix_timer->timer_on == 0); }

/*---------------------------------------------------------------------------*/
/*                                                                   */
/*---------------------------------------------------------------------------*/

/*
 * ؿƬ Pause, Unpause ǤФʤɤϡ
 * ؿؿƤӽФȤˡ Pause, Unpause Τ
 * դ뤳ȡ(ݡϥͥȤǤ)
 */

/*
 * ݡϥͥȤΤǡUNIXTimer_Unpause() °פ˷֤Ƥ
 * ʤ褦դ뤳ȡ
 */

/*
 * Τνϡ˥޳ߤȯʤɤ
 * դ뤳ȡǤʤԲĲʥХθˤʤ롥
 */

int UNIXTimer_Pause(UNIXTimer unix_timer)
{
  unix_timer->pause++; /* ݡϥͥȤǤ */

  return (0);
}

int UNIXTimer_Unpause(UNIXTimer unix_timer)
{
  if (unix_timer->pause == 0)
    Wonx_Error("UNIXTimer_Unpause", "Duplicated unpause.");

  if (unix_timer->pause == 1) {
    if (unix_timer->interrupt_in_pause > 0) {

      /*
       * 졤ǥ޳ߤȯƤ⡤
       * unix_timer->pause  1 ʤΤǡХåؿϸƤФʤΤǡ
       * ̵
       * ȥץꥻåȤʤС٤Ĥ̵
       * ȥץꥻåȤǤʤ timer_on ե饰Τǡ̵
       */

      /*
       * ˥޳ߤȯݤʤΤǡ
       * ޤ򤤤ä̵ˤ롥
       */
      alarm(0);

      /*
       * ͤѹľľǳߤݤʤΤǡ
       * ߤOFFˤƤǥȤ롥
       */
      /* ХåؿƤˡpause ե饰̵ˤ */
      unix_timer->pause--; /* ݡϥͥȤǤ */

      /* ХåؿθƤӽФ */
      UNIXTimer_CallBackFunction(0);
    }
  } else {
    unix_timer->pause--; /* ݡϥͥȤǤ */
  }

  return (0);
}

int UNIXTimer_IsPause(UNIXTimer unix_timer)
{ return (unix_timer->pause != 0); }

/*---------------------------------------------------------------------------*/
/* ȥץꥻå                                                          */
/*---------------------------------------------------------------------------*/

int UNIXTimer_SetAutoPreset(UNIXTimer unix_timer)
{ return (unix_timer->auto_preset = 1); }
int UNIXTimer_ResetAutoPreset(UNIXTimer unix_timer)
{ return (unix_timer->auto_preset = 0); }
int UNIXTimer_IsAutoPreset(UNIXTimer unix_timer)
{ return (unix_timer->auto_preset != 0); }

/*---------------------------------------------------------------------------*/
/* 󥿡Х                                                              */
/*---------------------------------------------------------------------------*/

int UNIXTimer_GetInterval(UNIXTimer unix_timer)
{ return (unix_timer->interval); }
int UNIXTimer_SetInterval(UNIXTimer unix_timer, int interval)
{
  /* ualarm(); ΰ0ѤƤϥʤΤ */
  if (interval < 1) interval = 1;
  return (unix_timer->interval = interval);
}

/*---------------------------------------------------------------------------*/
/* ХåؿθƤӽФΥѥ᡼                                  */
/*---------------------------------------------------------------------------*/

void * UNIXTimer_GetParameter(UNIXTimer unix_timer)
{ return (unix_timer->parameter); }
void * UNIXTimer_SetParameter(UNIXTimer unix_timer, void * parameter)
{ return (unix_timer->parameter = parameter); }

/*---------------------------------------------------------------------------*/
/* ХåؿμϿ                                              */
/*---------------------------------------------------------------------------*/

UNIXTimerCallBack UNIXTimer_GetCallBack(UNIXTimer unix_timer)
{ return (unix_timer->callback); }
UNIXTimerCallBack UNIXTimer_SetCallBack(UNIXTimer unix_timer,
					UNIXTimerCallBack callback)
{ return (unix_timer->callback = callback); }

/*---------------------------------------------------------------------------*/
/* ֥Ȥκ                                                        */
/*---------------------------------------------------------------------------*/

UNIXTimer UNIXTimer_Create(int auto_preset, int interval, void * parameter,
			   UNIXTimerCallBack callback)
{
  UNIXTimer unix_timer;

  unix_timer = (UNIXTimer)malloc(sizeof(_UNIXTimer));
  if (unix_timer == NULL)
    Wonx_Error("UNIXTimer_Create", "Cannot allocate memory.");

  unix_timer->timer_on           = 0;
  unix_timer->pause              = 0;
  unix_timer->interrupt_in_pause = 0;
  unix_timer->auto_preset        = 0;
  unix_timer->interval           = 0;
  unix_timer->parameter          = NULL;
  unix_timer->callback           = NULL;

  if (auto_preset) UNIXTimer_SetAutoPreset(unix_timer);
  else UNIXTimer_ResetAutoPreset(unix_timer);

  UNIXTimer_SetInterval(unix_timer, interval);
  UNIXTimer_SetParameter(unix_timer, parameter);
  UNIXTimer_SetCallBack(unix_timer, callback);

  /*
   * SIGALRM ѤΤǡsleep() ˥顼बˤϡ
   * Хåؿ sleep() λĤ֤Ϸ³ʤ
   */
  signal(SIGALRM, UNIXTimer_CallBackFunction);

  return (unix_timer);
}

/*---------------------------------------------------------------------------*/
/* ֥Ȥκ                                                        */
/*---------------------------------------------------------------------------*/

/*
 * Τνϡ˥޳ߤȯʤɤ
 * դ뤳ȡǤʤԲĲʥХθˤʤ롥
 */

UNIXTimer UNIXTimer_Destroy(UNIXTimer unix_timer)
{
  if (unix_timer == NULL)
    Wonx_Error("UNIXTimer_Destroy", "Object is not created.");

  UNIXTimer_OFF(unix_timer);

  free(unix_timer);

  return (NULL);
}

/*****************************************************************************/
/* ޤ                                                                  */
/*****************************************************************************/

/*****************************************************************************/
/* End of File.                                                              */
/*****************************************************************************/

==> v0.5-beta.tar.gz: wonx/UNIXTimer.c <==
/*****************************************************************************/
/*                                                                   */
/*****************************************************************************/

#include <unistd.h>
#include <signal.h>

#include "UNIXTimerP.h"
#include "etc.h"

/*****************************************************************************/
/* дؿ                                                          */
/*****************************************************************************/

/*---------------------------------------------------------------------------*/
/* Хåؿ                                                          */
/*---------------------------------------------------------------------------*/

/* pointed_unix_timer ͤϤѲ뤫狼ʤΤǡŬػߤ */

volatile static UNIXTimer pointed_unix_timer = NULL;

static void UNIXTimer_CallBackFunction(int argument)
{
  int ret;

  /*
   * static ʥե饰ΩƤơХåؿ饳Хåؿ
   * ƤФ줿Τ򸡽ФΤɬפ⡥
   */

  if (pointed_unix_timer == NULL) return;
  if (!pointed_unix_timer->timer_on) return;

  if (pointed_unix_timer->pause) {
    pointed_unix_timer->interrupt_in_pause++;
  } else {
    if (pointed_unix_timer->interrupt_in_pause == 0)
      pointed_unix_timer->interrupt_in_pause = 1;
    while (pointed_unix_timer->interrupt_in_pause > 0) {
      pointed_unix_timer->interrupt_in_pause--;
      if (pointed_unix_timer->callback != NULL) {

	/*
	 * Хåؿ椫 UNIXTimer_Unpause() ʤɤƤФơ
	 * 餵˥ХåؿƤФ줿ꤷȤΤˡ
	 * ݡ롥
	 */
	pointed_unix_timer->pause++;

	ret = (*pointed_unix_timer->callback)(pointed_unix_timer->parameter);

	pointed_unix_timer->pause--;

	/*
	 * Хåؿ UNIXTimer_* ϢδؿƤӽФȡ
	 *  UNIXTimer ֥Ȥξ֤ѤäƤޤǽ롥
	 */

	/*
	 * ʲνƱȤؿƬǤäƤΤǡ
	 * Ŭƺʤ褦դ뤳ȡ
	 */
	if (pointed_unix_timer == NULL) return;
	if (!pointed_unix_timer->timer_on) return;

	if (ret) {
	  pointed_unix_timer->interrupt_in_pause = 0;
	  UNIXTimer_OFF(pointed_unix_timer);
	  return;
	}

	/*
	 * Хåؿǡݡξ֤ڤؤäƤޤä
	 * ȤΤᡥ
	 * ʲνƱ褦ʤȤäˤäƤΤǡ
	 * Ŭƺʤ褦դ뤳ȡ
	 */
	if (pointed_unix_timer->pause) break;

      }
    }
  }

  if (pointed_unix_timer->auto_preset)
    UNIXTimer_ON(pointed_unix_timer);
  else
    UNIXTimer_OFF(pointed_unix_timer);

  return;
}

/*---------------------------------------------------------------------------*/
/* ޤ ON, OFF                                                          */
/*---------------------------------------------------------------------------*/

/*
 * Τνϡ˥޳ߤȯʤɤ
 * դ뤳ȡǤʤԲĲʥХθˤʤ롥
 */

int UNIXTimer_ON(UNIXTimer unix_timer)
{
  int t;

  unix_timer->timer_on = 1;
  pointed_unix_timer = unix_timer;

  /*
   * SIGALRM ѤΤǡsleep() ˥顼बˤϡ
   * Хåؿ sleep() λĤ֤Ϸ³ʤ
   */
  signal(SIGALRM, UNIXTimer_CallBackFunction);

#if 0 /* use ualarm() */
  /* ualarm(); ΰ0ѤƤϥʤΤ */
  t = unix_timer->interval * 1000;
  if (t < 1) t = 1;
  ualarm(t, 0);
#else /* use alarm() */
  /* alarm(); ΰ0ѤƤϥʤΤ */
  t = unix_timer->interval / 1000;
  if (t < 1) t = 1;
  alarm(t);
#endif

  return (0);
}

int UNIXTimer_OFF(UNIXTimer unix_timer)
{
  alarm(0); /* ޤ̵ˤ */
  unix_timer->timer_on = 0;
  pointed_unix_timer = NULL;
  return (0);
}

int UNIXTimer_IsON(UNIXTimer unix_timer)
{ return (unix_timer->timer_on != 0); }
int UNIXTimer_IsOFF(UNIXTimer unix_timer)
{ return (unix_timer->timer_on == 0); }

/*---------------------------------------------------------------------------*/
/*                                                                   */
/*---------------------------------------------------------------------------*/

/*
 * ؿƬ Pause, Unpause ǤФʤɤϡ
 * ؿؿƤӽФȤˡ Pause, Unpause Τ
 * դ뤳ȡ(ݡϥͥȤǤ)
 */

/*
 * ݡϥͥȤΤǡUNIXTimer_Unpause() °פ˷֤Ƥ
 * ʤ褦դ뤳ȡ
 */

/*
 * Τνϡ˥޳ߤȯʤɤ
 * դ뤳ȡǤʤԲĲʥХθˤʤ롥
 */

int UNIXTimer_Pause(UNIXTimer unix_timer)
{
  unix_timer->pause++; /* ݡϥͥȤǤ */

  return (0);
}

int UNIXTimer_Unpause(UNIXTimer unix_timer)
{
  if (unix_timer->pause == 0)
    Wonx_Error("UNIXTimer_Unpause", "Duplicated unpause.");

  if (unix_timer->pause == 1) {
    if (unix_timer->interrupt_in_pause > 0) {

      /*
       * 졤ǥ޳ߤȯƤ⡤
       * unix_timer->pause  1 ʤΤǡХåؿϸƤФʤΤǡ
       * ̵
       * ȥץꥻåȤʤС٤Ĥ̵
       * ȥץꥻåȤǤʤ timer_on ե饰Τǡ̵
       */

      /*
       * ˥޳ߤȯݤʤΤǡ
       * ޤ򤤤ä̵ˤ롥
       */
      alarm(0);

      /*
       * ͤѹľľǳߤݤʤΤǡ
       * ߤOFFˤƤǥȤ롥
       */
      /* ХåؿƤˡpause ե饰̵ˤ */
      unix_timer->pause--; /* ݡϥͥȤǤ */

      /* ХåؿθƤӽФ */
      UNIXTimer_CallBackFunction(0);
    }
  } else {
    unix_timer->pause--; /* ݡϥͥȤǤ */
  }

  return (0);
}

int UNIXTimer_IsPause(UNIXTimer unix_timer)
{ return (unix_timer->pause != 0); }

/*---------------------------------------------------------------------------*/
/* ȥץꥻå                                                          */
/*---------------------------------------------------------------------------*/

int UNIXTimer_SetAutoPreset(UNIXTimer unix_timer)
{ return (unix_timer->auto_preset = 1); }
int UNIXTimer_ResetAutoPreset(UNIXTimer unix_timer)
{ return (unix_timer->auto_preset = 0); }
int UNIXTimer_IsAutoPreset(UNIXTimer unix_timer)
{ return (unix_timer->auto_preset != 0); }

/*---------------------------------------------------------------------------*/
/* 󥿡Х                                                              */
/*---------------------------------------------------------------------------*/

int UNIXTimer_GetInterval(UNIXTimer unix_timer)
{ return (unix_timer->interval); }
int UNIXTimer_SetInterval(UNIXTimer unix_timer, int interval)
{ return (unix_timer->interval = interval); }

/*---------------------------------------------------------------------------*/
/* ХåؿθƤӽФΥѥ᡼                                  */
/*---------------------------------------------------------------------------*/

void * UNIXTimer_GetParameter(UNIXTimer unix_timer)
{ return (unix_timer->parameter); }
void * UNIXTimer_SetParameter(UNIXTimer unix_timer, void * parameter)
{ return (unix_timer->parameter = parameter); }

/*---------------------------------------------------------------------------*/
/* ХåؿμϿ                                              */
/*---------------------------------------------------------------------------*/

UNIXTimerCallBack UNIXTimer_GetCallBack(UNIXTimer unix_timer)
{ return (unix_timer->callback); }
UNIXTimerCallBack UNIXTimer_SetCallBack(UNIXTimer unix_timer,
					UNIXTimerCallBack callback)
{ return (unix_timer->callback = callback); }

/*---------------------------------------------------------------------------*/
/* ֥Ȥκ                                                        */
/*---------------------------------------------------------------------------*/

UNIXTimer UNIXTimer_Create(int auto_preset, int interval, void * parameter,
			   UNIXTimerCallBack callback)
{
  UNIXTimer unix_timer;

  unix_timer = (UNIXTimer)malloc(sizeof(_UNIXTimer));
  if (unix_timer == NULL)
    Wonx_Error("UNIXTimer_Create", "Cannot allocate memory.");

  unix_timer->timer_on           = 0;
  unix_timer->pause              = 0;
  unix_timer->interrupt_in_pause = 0;
  unix_timer->auto_preset        = 0;
  unix_timer->interval           = 0;
  unix_timer->parameter          = NULL;
  unix_timer->callback           = NULL;

  if (auto_preset) UNIXTimer_SetAutoPreset(unix_timer);
  else UNIXTimer_ResetAutoPreset(unix_timer);

  UNIXTimer_SetInterval(unix_timer, interval);
  UNIXTimer_SetParameter(unix_timer, parameter);
  UNIXTimer_SetCallBack(unix_timer, callback);

  return (unix_timer);
}

/*---------------------------------------------------------------------------*/
/* ֥Ȥκ                                                        */
/*---------------------------------------------------------------------------*/

/*
 * Τνϡ˥޳ߤȯʤɤ
 * դ뤳ȡǤʤԲĲʥХθˤʤ롥
 */

UNIXTimer UNIXTimer_Destroy(UNIXTimer unix_timer)
{
  if (unix_timer == NULL)
    Wonx_Error("UNIXTimer_Destroy", "Object is not created.");

  UNIXTimer_OFF(unix_timer);

  free(unix_timer);

  return (NULL);
}

/*****************************************************************************/
/* ޤ                                                                  */
/*****************************************************************************/

/*****************************************************************************/
/* End of File.                                                              */
/*****************************************************************************/
